/**
 * Port from https://github.com/stacktracejs/error-stack-parser-es
 */
interface StackFrameOptions {
    isConstructor?: boolean;
    isEval?: boolean;
    isNative?: boolean;
    isToplevel?: boolean;
    columnNumber?: number;
    lineNumber?: number;
    fileName?: string;
    functionName?: string;
    source?: string;
    args?: any[];
    evalOrigin?: StackFrame;
}
interface StackFrame {
    new (obj: StackFrameOptions): StackFrame;
    args?: any[];
    getArgs(): any[] | undefined;
    setArgs(args: any[]): void;
    evalOrigin?: StackFrame;
    getEvalOrigin(): StackFrame | undefined;
    setEvalOrigin(stackframe: StackFrame): void;
    isConstructor?: boolean;
    getIsConstructor(): boolean | undefined;
    setIsConstructor(isConstructor: boolean): void;
    isEval?: boolean;
    getIsEval(): boolean | undefined;
    setIsEval(isEval: boolean): void;
    isNative?: boolean;
    getIsNative(): boolean | undefined;
    setIsNative(isNative: boolean): void;
    isToplevel?: boolean;
    getIsToplevel(): boolean | undefined;
    setIsToplevel(isToplevel: boolean): void;
    columnNumber?: number;
    getColumnNumber(): number | undefined;
    setColumnNumber(columnNumber: number): void;
    lineNumber?: number;
    getLineNumber(): number | undefined;
    setLineNumber(lineNumber: number): void;
    fileName?: string;
    getFileName(): string | undefined;
    setFileName(fileName: string): void;
    functionName?: string;
    getFunctionName(): string | undefined;
    setFunctionName(functionName: string): void;
    source?: string;
    getSource(): string | undefined;
    setSource(source: string): void;
    toString(): string;
}
/**
 * Given an Error object, extract the most information from it.
 *
 * @param {Error} error object
 * @return {Array} of StackFrames
 */
declare function parse(error: Error): StackFrame[];
declare function extractLocation(urlLike: string): string[] | readonly [string, string | undefined, string | undefined];
declare function parseV8OrIE(error: Error): StackFrame[];
declare function parseFFOrSafari(error: Error): StackFrame[];
declare function parseOpera(e: Error): StackFrame[];
declare function parseOpera9(e: Error): StackFrame[];
declare function parseOpera10(e: Error): StackFrame[];
declare function parseOpera11(error: Error): StackFrame[];

export { StackFrame, StackFrameOptions, extractLocation, parse, parseFFOrSafari, parseOpera, parseOpera10, parseOpera11, parseOpera9, parseV8OrIE };
